#
#  AppController.rb
#  Metadata Mover
#
#  Created by Gregory Clarke on 2/28/10.
#  Copyright (c) 2010 Intelligent Assistance, Inc. All rights reserved.
#

require 'osx/cocoa'
require 'copy_metadata.bundle'

class AppController < OSX::NSObject

	ib_outlets :mainWindow, :deactivateMenuItem
	ib_outlets :theView, :xmlIcon, :xmlLabel, :fromBinPopup, :toBinPopup, :saveXMLButton, :spinner
	ib_outlets :regoView, :deactivateAreaView, :webView, :problemMessage, :closeRegoButton

	def awakeFromNib
		# will use CoreAnimation for the window changes:
		@mainWindow.contentView.wantsLayer = true
		@xmlIcon.setHidden(true)
		@xmlIcon.setToolTip("")
		@xmlLabel.setStringValue("")
	end

	### rego methods ###
	
	def applicationWillFinishLaunching(sender) # applicationWillFinishLaunching: is sent before application:openFile: (so it can deal with a file dropped onto the app icon)
		puts "applicationWillFinishLaunching"
		check_for_expired_beta()
		if @this_rego[0]["active"]
			showMainView()
		elsif @this_rego[0]["mac_problem"]==true
			check_for_rego(@this_rego)
		else
			@deactivateMenuItem.setTitle("Activate...")
			displayDemoSheet()
		end
	end
	
	def displayDemoSheet
		alert = OSX::NSAlert.alloc.init
		alert.setMessageText("#{SKU} has not been activated for this computer.")
		alert.setAlertStyle(OSX::NSInformationalAlertStyle)
		alert.setInformativeText("Would you like to activate #{SKU}, or try it in demo mode (which is limited to random clips)?")
		alert.addButtonWithTitle("Activate")
		alert.addButtonWithTitle("Buy")
		alert.addButtonWithTitle("Demo")
		alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, "didEndDemoSheet:returnCode:contextInfo:", nil)
	end
	
	def didEndDemoSheet_returnCode_contextInfo(sheet, buttonClicked, info)
		sheet.window.orderOut(nil)
		if buttonClicked==OSX::NSAlertSecondButtonReturn
			system('open', 'http://www.AssistedEditing.com/LogNoteReplicator/buy')
		end
		if buttonClicked==OSX::NSAlertThirdButtonReturn
			showMainView()
		else
			newFrame = self.newFrameForNewContentView(@regoView)
			showActivationView(newFrame, WEB_APP + "license/" + SKU.gsub(" ", "%20") + "?sn=" + @this_rego[0]["number"].to_s + "&mac=" + MAC) # NEW
		end
	end
	
	def showDeactivation
		if @deactivateMenuItem.title=="Activate..."
			newFrame = self.newFrameForNewContentView(@regoView)
			showActivationView(newFrame, WEB_APP + "license/" + SKU.gsub(" ", "%20") + "?sn=" + @this_rego[0]["number"].to_s + "&mac=" + MAC) # NEW
		else
			newFrame = self.newFrameForNewContentView(@deactivateAreaView) # 500 x 200
			@mainWindow.setMinSize(OSX::NSSize.new(500, 200+22))
			@mainWindow.setMaxSize(OSX::NSSize.new(500, 200+22))
			@mainWindow.setContentView(@deactivateAreaView)
			showActivationView(newFrame, WEB_APP + "destroy/" + @this_rego[0]["number"])
			@closeRegoButton.setTitle("Cancel")
		end
	end
	ib_action :showDeactivation # First Cuts > Deactivate
	
	def showActivationView(frame, url)
		puts "showActivationView #{url}"
		@mainWindow.setFrame_display frame, true
		@mainWindow.setContentView(@regoView)
		@mainWindow.setTitle("Activation")
		# notification
		center = OSX::NSNotificationCenter.defaultCenter
		center.addObserver_selector_name_object(self, :webViewProgressFinished, "WebProgressFinishedNotification", @webView) # name is WebProgressFinishedNotification NOT WebViewProgressFinishedNotification
		@webView.mainFrame.loadRequest( OSX::NSURLRequest.requestWithURL( OSX::NSURL.URLWithString(url) ) )
	end
	
	def webViewProgressFinished(notification)
		puts notification.name
		if notification.name=="WebProgressFinishedNotification"
			if @webView.mainFrame.dataSource.nil? # in case page doesn't load...
				@webView.mainFrame.stopLoading
				@problemMessage.setHidden(false)
				@webView.setHidden(true)
			else
				get_web_app_response(@webView.mainFrame.dataSource.response.URL)
			end
		end
	end 
	
	def closeActivationView
		if @closeRegoButton.title=="Continue" || @closeRegoButton.title=="Cancel"
			showMainView()
		else
			OSX::NSApplication.sharedApplication.terminate(self)
		end
	end
	ib_action :closeActivationView # on click of Registration window's Continue button
	
	### end rego methods ###
	
	def showMainView
		setupView(@theView, "Log Note Replicator", 436, 251)
		@xmlLabel.setStringValue("")
	end
	
	def setupView(view, title, width, height)
		@mainWindow.setTitle(title)
		oldFrame = @mainWindow.frame
		newFrame = self.newFrameForNewContentView(view)
		@mainWindow.setMinSize(OSX::NSSize.new(width, height+22))
		@mainWindow.setMaxSize(OSX::NSSize.new(width, height+22))
		# Using an animation grouping because we may be changing the duration
		OSX::NSAnimationContext.beginGrouping
			# Call the animator instead of the view / window directly
			@mainWindow.animator.setFrame_display newFrame, true
		OSX::NSAnimationContext.endGrouping
		@mainWindow.setContentView(view)
	end

	def newFrameForNewContentView(view)
		newFrameRect = @mainWindow.frameRectForContentRect(view.frame)
		oldFrameRect = @mainWindow.frame
		newSize = newFrameRect.size
		oldSize = oldFrameRect.size
		frame = @mainWindow.frame
		frame.size = newSize
		frame.origin.y = frame.origin.y - (newSize.height - oldSize.height)
		frame
	end
	
	def showHelp(sender)
		openPDF("Log Note Replicator - Help.pdf")
	end
	ib_action :showHelp # Help > miniME Help
	
	def openPDF(filename)
		path_to_file = __FILE__.sub("AppController.rb",filename)
		system('open', path_to_file)
	end

	def visitBuyPage(sender)
		system('open', 'http://www.AssistedEditing.com/LogNoteReplicator/buy')
	end
	ib_action :visitBuyPage # Help > Visit www.AssistedEditing.com

	def visitWebsite(sender)
		system('open', 'http://www.AssistedEditing.com/')
	end
	ib_action :visitWebsite # Help > Visit www.AssistedEditing.com
	
	def sendFeedback(sender)
		OSX::JRFeedbackController.showFeedback
	end
	ib_action :sendFeedback # Application > Provide Feedback

	def chooseFinalCutProXML(sender)
		filetypes = ["xml"]
		oPanel = OSX::NSOpenPanel.openPanel
		oPanel.setTitle("Choose the Final Cut Pro XML file")
		oPanel.setAllowsMultipleSelection(false)
		oPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, filetypes, @mainWindow, self, 'didEndOpenFCPXMLSheet:returnCode:contextInfo:', nil)
	end
	ib_action :chooseFinalCutProXML

	def didEndOpenFCPXMLSheet_returnCode_contextInfo(oPanel, buttonClicked, info)
		if buttonClicked == OSX::NSOKButton
			oPanel.orderOut(self)
			file = oPanel.filenames.objectAtIndex(0)
			@xmlLabel.setStringValue("")
			@xmlIcon.setHidden(true)
			@fromBinPopup.removeAllItems
			@toBinPopup.removeAllItems
			@fromBinPopup.setEnabled(false)
			@toBinPopup.setEnabled(false)
			@saveXMLButton.setEnabled(false)
			application_setXmlFile(self, file)
		end
	end

	def application_openFile(sender, path)
		OSX::NSDocumentController.sharedDocumentController().noteNewRecentDocumentURL( OSX::NSURL.fileURLWithPath(path) ) # add this file to the Recent menu
		# fileManager = OSX::NSFileManager.defaultManager
		chosen_file = path.to_s
		build_bins_popup_menus(chosen_file)
	end

	def addMetadataToClips(sender)
		@spinner.startAnimation(self)
		save_temp_xml_file()
		@spinner.stopAnimation(self)
	end
	ib_action :addMetadataToClips

	def applicationShouldTerminateAfterLastWindowClosed(sender)
		true
	end

end
